<?php

declare(strict_types=1);

namespace App\Controller\OgAgent;

use App\Entity\Client;
use App\Entity\Command;
use App\Entity\Image;
use App\Entity\Trace;
use App\Model\ClientStatus;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Validator\Exception\ValidatorException;
use Symfony\Contracts\HttpClient\Exception\ClientExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\RedirectionExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\ServerExceptionInterface;
use Symfony\Contracts\HttpClient\Exception\TransportExceptionInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

class CreateImageActionController extends AbstractController
{
    public function __construct(
        protected readonly EntityManagerInterface $entityManager,
        protected readonly HttpClientInterface    $httpClient
    )
    {
    }


    public function __invoke(Image $image): JsonResponse
    {
        if (!$image->getClient()->getIp()) {
            throw new ValidatorException('IP is required');
        }

        $partitionInfo = json_decode($image->getPartitionInfo(), true);

        $data = [
            'dsk' => (string) $partitionInfo['numDisk'],
            'par' => (string) $partitionInfo['numPartition'],
            'cpt' => "83",
            'idi' => $image->getUuid(),
            'nci' => $image->getName(),
            'ipr' => $image->getRepository()->getIp(),
            'nfn' => 'CrearImagen',
            'ids' => '0'
        ];

        try {
            $response = $this->httpClient->request('POST', 'https://localhost:4444/CloningEngine/CrearImagen', [
                'verify_peer' => false,
                'verify_host' => false,
                'headers' => [
                    'Content-Type' => 'application/json',
                ],
                'json' => $data,
            ]);

        } catch (TransportExceptionInterface $e) {
            return new JsonResponse(
                data: ['error' => $e->getMessage()],
                status: Response::HTTP_INTERNAL_SERVER_ERROR
            );
        }

        $jobId = json_decode($response->getContent(), true)['job_id'];

        $command = $this->entityManager->getRepository(Command::class)->findOneBy(['name' => 'Crear Imagen']);

        $client = $image->getClient();
        $client->setStatus(ClientStatus::BUSY);
        $this->entityManager->persist($client);

        $trace = new Trace();
        $trace->setClient($image->getClient());
        $trace->setCommand($command ?? null);
        $trace->setStatus('pending');
        $trace->setJobId($jobId);
        $trace->setExecutedAt(new \DateTime());

        $this->entityManager->persist($trace);
        $this->entityManager->flush();

        return new JsonResponse(status: Response::HTTP_OK);
    }
}
